/*
 * PCF no-repad path: glyph extent past bitmap buffer
 *
 * The font stores bitmaps at the same glyph pad as the server requests
 * (pad=4), so the RepadBitmap path is skipped.  The original validation
 * only checked that the glyph's offset was within the buffer, but not
 * that the glyph's full extent (offset + width*height) fits.
 *
 * With the fix, the full glyph size is validated against the buffer.
 *
 * Test font: test/data/norepad/evil-norepad.pcf
 * Generated by: test/gen_evil_pcf_norepad.py
 */

/*
 * Copyright (c) 2026, Red Hat, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "pcf-test-common.h"
#include <stdio.h>
#include <stdlib.h>

int
main(int argc, char **argv)
{
    xfont2_fpe_funcs_rec const **fpe_functions;
    int fpe_function_count;
    const char *builddir;
    char fontdir[512];

    builddir = getenv("builddir");
    if (!builddir)
        builddir = ".";

    fpe_functions = pcf_test_init(&fpe_function_count);

    snprintf(fontdir, sizeof(fontdir), "%s/test/data/norepad", builddir);
    return pcf_test_open_font(fpe_functions,
                              "pcf-norepad-oob",
                              fontdir,
                              "-evil-norepad-medium-r-normal--10-100-75-75-c-80-iso10646-1");
}
